/* include/openh323buildopts.h.  Generated from openh323buildopts.h.in by configure.  */
/*
 * openh323buildopts.h
 *
 * Build options generated by the configure script.
 *
 * Portable Windows Library
 *
 * Copyright (c) 2003     Equivalence Pty. Ltd.
 * Copyright (c) 20007-13 ISVO(Asia) Pte Ltd.
 * 
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Contributor(s): ______________________________________.
 *
 * $Id$
 *
 */

#ifndef _OPENH323_BUILDOPTS_H
#define _OPENH323_BUILDOPTS_H

// PTLIB VERSION
#define PT_MAJOR_VERSION 2
#define PT_MINOR_VERSION 10
#define PT_BUILD_NUMBER 9

#if PT_MAJOR_VERSION == 2 && PT_MINOR_VERSION >= 13
#include <ptlib_config.h>
#else
#include <ptbuildopts.h>
#endif

#if PTLIB_MAJOR == 2 && PTLIB_MINOR < 10
   #define PTLIB_VER ( PTLIB_MAJOR*100 + PTLIB_MINOR*10 + PTLIB_BUILD )
#else
   #if (PTLIB_BUILD < 10)  // We currently only handle Build numbers less then 10 - SH
      #define PTLIB_VER ( PTLIB_MAJOR*1000 + PTLIB_MINOR*10 + PTLIB_BUILD )
   #else
      #define PTLIB_VER ( PTLIB_MAJOR*1000 + PTLIB_MINOR*10 + 9 )
   #endif 
#endif


/////////////////////////////////////////////////
//
// Major options
//

#define OPENH323_MAJOR 1
#define OPENH323_MINOR 28
#define OPENH323_BUILD 0
#define OPENH323_VERSION "1.28.0"
#define H323PLUS_VER ( OPENH323_MAJOR*1000 + OPENH323_MINOR*10 + OPENH323_BUILD )


#if PTLIB_VER >= 2110
  #define H323_STLDICTIONARY  1
#endif

#define H323_AUDIO_CODECS 1

#ifdef P_VIDEO
#define H323_VIDEO 1
#endif

#ifdef P_SSL
#define H323_SSL 1
#endif


//////////////////////////////////////////////////
// static codec definitions

#ifdef H323_AUDIO_CODECS
/*    #undef H323_STATIC_G7221 */
/*    #undef H323_STATIC_GSM */
#endif

#ifdef H323_VIDEO
/*    #undef H323_STATIC_H261 */
/*    #undef H323_STATIC_H263 */
/*    #undef H323_STATIC_H264 */
#endif

/////////////////////////////////////////////////
//
// Various item support
//

#define H323_T38 1
/* #undef H323_T120 */
/* #undef H323_T140 */
#define H323_H224 1
#ifdef H323_H224
    #define H224_H281   1
/*     #undef H224_T140 */
/*     #undef H224_H284 */
#endif
#define H323_H230 1
#if (PTLIB_VER > 260) && defined(H323_SSL)
  #define H323_H235 1
  #ifdef H323_H235
/*     #undef H323_H235_AES256 */
  #endif
  #define H323_TLS 1
#endif
#ifdef P_VIDEO
#define H323_H239 1
#endif
#define H323_H248 1
//#undef H323_H249
#ifdef P_SNMP
#define H323_H341 1
#endif
#ifdef P_LDAP
#define H323_H350 1
#endif
#define H323_H450 1

#define H323_H460 1
#ifdef H323_H460
#define H323_H4609 1
#if (PTLIB_VER > 260) && defined(P_STUN)
/*     #undef H323_H46017 */
    #if defined(H323_H46017)
/*         #undef H323_H46026 */
    #endif
/*     #undef H323_H46018 */
    #if defined(H323_H46018)
        #define H323_H46019M 1
    #endif
/*     #undef H323_H46023 */
    #if defined(H323_H46018) && defined(H323_H46023) && defined(H323_SSL)
        #define H323_H46024A 1
        #define H323_H46024B 1
    #endif
    #if defined(H323_H46023) && (WINVER > 0x500)
        #define H323_UPnP 1
    #endif
#endif

#ifdef P_EXPAT
/*     #undef H323_H46025 */
#endif

// Text message option
/* #undef H323_H460IM */

// Presence Feature
/* #undef H323_H460P */
#ifdef H323_H460P
   #define H323_H460P_VER 3
#endif
// Registration Priority Feature
/* #undef H323_H460PRE */
// Endpoint Compatibility 
/* #undef H323_H460COM */
#endif

// ASSET Feature
/* #undef H323_H461 */

#define H323_H501 1
#if (PTLIB_VER > 260) && (P_STUN)
    #define H323_GNUGK 1
#endif
#define H323_FILE 1

#ifndef _WIN64
/* #undef H323_AEC */
#endif
#if defined(_MSC_VER) && defined(H323_AEC)
#pragma include_alias(<speex/speex_echo.h>,            <@AEC_DIR@/include/speex/speex_echo.h>      )
#pragma include_alias(<speex/speex_preprocess.h>,      <@AEC_DIR@/include/speex/speex_preprocess.h>)
#pragma include_alias(<speex/speex_types.h>,           <@AEC_DIR@/include/speex/speex_types.h>     )
#define H323_AEC_LIB "@AEC_DIR@/lib/libspeexdsp.lib"
#endif

/* #undef H323_SIGNAL_AGGREGATE */
/* #undef H323_RTP_AGGREGATE */

/* #undef H323_FIXED_VIDEOCLOCK */

#define H323_FRAMEBUFFER 1

/* #undef H323_PACKET_TRACE */

#if PTRACING
/*     #undef H323_JITTER_ANALYSER */
#endif

#if (PTLIB_VER >= 2110)
/*  #undef H323_MEDIAENCODED */
#endif

#if P_HAS_IPV6
#define H323_IPV6   1
#endif

#endif

// End Of File ///////////////////////////////////////////////////////////////
