//
// h350_service.h
//
// Code automatically generated by asnparse.
//

#if ! H323_DISABLE_H225

#ifndef __H225_H350_H
#define __H225_H350_H

#ifdef P_USE_PRAGMA
#pragma interface
#endif

#include <ptclib/asner.h>

//
// H350ServiceControl
//

class H225_H350ServiceControl : public PASN_Sequence
{
#ifndef PASN_LEANANDMEAN
    PCLASSINFO(H225_H350ServiceControl, PASN_Sequence);
#endif
  public:
    H225_H350ServiceControl(unsigned tag = UniversalSequence, TagClass tagClass = UniversalTagClass);

    PASN_IA5String m_ldapURL;
    PASN_IA5String m_ldapDN;

    PINDEX GetDataLength() const;
    PBoolean Decode(PASN_Stream & strm);
    void Encode(PASN_Stream & strm) const;
#ifndef PASN_NOPRINTON
    void PrintOn(ostream & strm) const;
#endif
    Comparison Compare(const PObject & obj) const;
    PObject * Clone() const;
};


#endif // __H225_H

#endif // if ! H323_DISABLE_H225


// End of h350_service.h
